@@ -1,77 +1,10 @@
 @enum AnnotationFill BLANK=0 ABOVE=1 BELOW=2 EMPTY=3
 
-#############################################################
-##                     SegmentEvent                        ##
-#############################################################
-
-mutable struct SegmentAnnotations
-    fill_above::Union{Nothing, Bool}
-    fill_below::Union{Nothing, Bool}
-end
-
-SegmentAnnotations() = SegmentAnnotations(nothing, nothing)
-
-==(ann1::SegmentAnnotations, ann2::SegmentAnnotations) = 
-    (ann1.fill_above == ann2.fill_above) && (ann1.fill_below == ann2.fill_below)
-
-mutable struct SegmentEvent{T}
-    segment::Segment2D{T}
-    is_start::Bool
-    primary::Bool # primary or secondary polygon
-    self_annotations::SegmentAnnotations # this polygon
-    other_annotations::SegmentAnnotations # other polygon
-    # convenience properties
-    other::Union{Nothing,SegmentEvent{T}} # links to opposite event
-    point::Point2D{T} # is_start ? segment[1] : segment[2]
-    other_point::Point2D{T} # is_start ? segment[2] : segment[1]
-end
-
-function SegmentEvent(
-    segment::Segment2D,
-    is_start::Bool,
-    primary::Bool=true,
-    self_annotations::SegmentAnnotations=SegmentAnnotations(),
-    other_annotations::SegmentAnnotations=SegmentAnnotations(),
+"""
+    martinez_rueda_algorithm(
+    selection_criteria, subject, others...
+    ; atol=default_atol, rtol=default_rtol, fuzzy_rtol=1.0
     )
-    point = is_start ? segment[1] : segment[2]
-    other_point = is_start ? segment[2] : segment[1]
-    SegmentEvent(segment, is_start, primary, self_annotations, other_annotations, nothing, point, other_point)
-end
-
-function copy_segment(event::SegmentEvent) 
-    # make a copy independent of the original event.
-    SegmentEvent(
-        deepcopy(event.segment), event.is_start, event.primary, deepcopy(event.self_annotations), deepcopy(event.other_annotations)
-    )
-end
-
-"""
-    ==(ev1::SegmentEvent, ev2::SegmentEvent)
-
-Test equality by comparing attributes but excluding the `other` link.
-Use `===` instead if the `other` link is required.
-"""
-==(ev1::SegmentEvent, ev2::SegmentEvent) = 
-    (ev1.segment == ev2.segment) && (ev1.is_start == ev2.is_start) &&
-    (ev1.primary == ev2.primary) &&
-    (ev1.self_annotations == ev2.self_annotations) && (ev1.other_annotations == ev2.other_annotations)
-
-function Base.show(io::IO, event::SegmentEvent)
-    #print(io, typeof(event), "(")
-    print(io, "SegmentEvent(")
-    print(io, event.segment)
-    print(io, ", ", event.is_start)
-    print(io, ", ", event.primary)
-    print(io, ", ", event.self_annotations)
-    print(io, ", ", event.other_annotations)
-    #print(io, ", ", event.other)
-    #print(io, ", ", event.point)
-    #print(io, ", ", event.other_point)
-    print(io, ")")
-end
-
-"""
-    martinez_rueda_algorithm(polygon1, polygon2, selection_criteria)
 
 The Martínez-Rueda-Feito polygon clipping algorithm.
 Returns regions and edges of intersection.
@@ -79,18 +12,31 @@ It runs in `O((n+m+k)log(n+m))` time where `n` and `m` are the number of vertice
 and `polygon2` respectively and `k` is the total number of intersections between all segments.
 Use `intersect_convex` for convex polygons for an `O(n+m)` algorithm.
 
+The input polygons can be:
+- A list of points: `Vector{Tuple{Float64, Float64}}`.
+- Polygons: `Vector{Polygon{T}}`.
+- A Segment event queue: `Vector{<:SegmentEvent{Float64}}`. The core algorithm uses this representation.
+- The `subject` can also be a list of polygons. This enables the output to be an input. 
+    Note that in this case the `subject` list is treated as a single polygon.
+    If any of the polygons overlap, this is equivalent to passing a self-intersecting polygon and
+    some areas might be classified as holes according to the even-odd rule
+
 Description:
 - Operates at a segment level and is an extension of the Bentley-Ottman line intersection algorithm.
-Segments are scanned from left to right, bottom to top. 
+    Segments are scanned from left to right, bottom to top. 
 - The key assumption is that only the segments immediately above and below the current segment need to be inspected for intersections.
-This makes the algorithm fast but also sensitive to determining these segments correctly.
+    This makes the algorithm fast but also sensitive to determining these segments correctly.
 - The segment that is immediately below (or empty space) is used to determine the fill annotations for the current segment.
 - Once all annotations are done, the desired segments can be selected that match a given criteria.
+- These segments are then chained together to form the polygons.
 
 Limitations
 1. It can fail for improper polygons: polygons with lines sticking out.
 2. It is sensitive to numeric inaccuracies e.g. a line that is almost vertical or tiny regions 
-of intersection.
+    of intersection.
+3. Sometimes the segment chaining can fail. This might happen if the polygons are improper.
+    For some cases when the segment chaining fails it is possible to "fuzzy" close them.
+    The criteria is that the ratio of the remaining gap to the mean segment length is less than `fuzzy_rtol`.
 
 References 
 - paper: https://www.researchgate.net/publication/220163820_A_new_algorithm_for_computing_Boolean_operations_on_polygons
@@ -98,40 +44,111 @@ References
 - article source code: https://github.com/velipso/polybooljs
 """
 function martinez_rueda_algorithm(
-    polygon1::Polygon2D{T},
-    polygon2::Polygon2D{T},
-    selection_criteria::Vector{AnnotationFill}
-    ; atol::AbstractFloat=1e-6, rtol::AbstractFloat=1e-4
+    selection_criteria::Vector{AnnotationFill},
+    subject::Path2D{T},
+    others::Vararg{Path2D{T}},
+    ; atol::AbstractFloat=default_atol, options...
     ) where T
-    event_queue1 = convert_to_event_queue(polygon1; primary=true, atol=atol)
-    event_queue2 = convert_to_event_queue(polygon2; primary=false, atol=atol)
-    martinez_rueda_algorithm(event_queue1, event_queue2, selection_criteria; atol=atol, rtol=rtol)
+    event_queue_base = convert_to_event_queue(subject; primary=true, atol=atol)
+    event_queue_others = map(p -> convert_to_event_queue(p; primary=false, atol=atol), others)
+    region_segments = martinez_rueda_algorithm(
+        selection_criteria, event_queue_base, event_queue_others...; atol=atol, options...
+    )
+    map(segments -> map(event -> event.point, segments), region_segments)
 end
 
 function martinez_rueda_algorithm(
-    event_queue1::Vector{<:SegmentEvent{T}},
-    event_queue2::Vector{<:SegmentEvent{T}},
-    selection_criteria::Vector{AnnotationFill}
-    ; atol::AbstractFloat=1e-6, rtol::AbstractFloat=1e-4
+    selection_criteria::Vector{AnnotationFill},
+    subjects::AbstractVector{<:Path2D{T}},
+    others::Vararg{Path2D{T}},
+    ; atol::AbstractFloat=default_atol, options...
     ) where T
-    annotated_segments1 = event_loop!(event_queue1; self_intersection=true, atol=atol, rtol=rtol)
-    annotated_segments2 = event_loop!(event_queue2; self_intersection=true, atol=atol, rtol=rtol)
-    queue = SegmentEvent{T}[]
-    for ev in vcat(annotated_segments1, annotated_segments2)
-        add_annotated_segment!(queue, ev)
+    subject_queue = SegmentEvent{T}[]
+    map(p -> convert_to_event_queue!(subject_queue, p; primary=true, atol=atol), subjects)
+    event_queue_others = map(p -> convert_to_event_queue(p; primary=false, atol=atol), others)
+    region_segments = martinez_rueda_algorithm(
+        selection_criteria, subject_queue, event_queue_others...; atol=atol, options...
+    )
+    map(segments -> map(event -> event.point, segments), region_segments)
+end
+
+# Polygon with hole input
+
+function martinez_rueda_algorithm(
+    selection_criteria::Vector{AnnotationFill},
+    subject::Polygon{T},
+    others::Vararg{Polygon{T}},
+    ; atol::AbstractFloat=default_atol, options...
+    ) where T
+    event_queue_base = convert_to_event_queue(subject.exterior; primary=true, atol=atol)
+    for hole in subject.holes
+        convert_to_event_queue!(event_queue_base, hole; primary=true, atol=atol)
     end
-    annotated_segments3 = event_loop!(queue; self_intersection=false, atol=atol, rtol=rtol)
-    # for consistent reporting, swap annotations so that self annotations are always the primary
-    for ev in annotated_segments3
-        if !ev.primary
-            temp = ev.self_annotations
-            ev.self_annotations = ev.other_annotations
-            ev.other_annotations = temp
+    event_queue_others = map(p -> convert_to_event_queue(p.exterior; primary=false, atol=atol), others)
+    for (queue, other) in zip(event_queue_others, others)
+        for hole in other.holes
+            convert_to_event_queue!(queue, hole; primary=false, atol=atol)
         end
     end
-    selected = apply_selection_criteria(annotated_segments3, selection_criteria)
-    empty_segments, regions_segments = separate(is_empty_segment, selected)
-    regions = chain_segments(regions_segments; atol=atol, check_closes=true)
+    region_segments = martinez_rueda_algorithm(
+        selection_criteria, event_queue_base, event_queue_others...; atol=atol, options...
+    )
+    convert_segments_to_polygons(region_segments; atol=atol)
+end
+
+function martinez_rueda_algorithm(
+    selection_criteria::Vector{AnnotationFill},
+    subjects::AbstractVector{<:Polygon{T}},
+    clips::Vararg{Polygon{T}},
+    ; atol::AbstractFloat=default_atol, options...
+    ) where T
+    subject_queue = SegmentEvent{T}[]
+    map(p -> convert_to_event_queue!(subject_queue, p.exterior; primary=true, atol=atol), subjects)
+    for subject in subjects
+        for hole in subject.holes
+            convert_to_event_queue!(subject_queue, hole; primary=true, atol=atol)
+        end
+    end
+    event_queue_clips = map(p -> convert_to_event_queue(p.exterior; primary=false, atol=atol), clips)
+    for (queue, other) in zip(event_queue_clips, clips)
+        for hole in other.holes
+            convert_to_event_queue!(queue, hole; primary=false, atol=atol)
+        end
+    end
+    region_segments = martinez_rueda_algorithm(
+        selection_criteria, subject_queue, event_queue_clips...; atol=atol, options...
+    )
+    convert_segments_to_polygons(region_segments; atol=atol)
+end
+
+# Core algorithm: SegmentEvent input
+
+function martinez_rueda_algorithm(
+    selection_criteria::Vector{AnnotationFill},
+    subject::Vector{<:SegmentEvent{T}},
+    polygons::Vararg{Vector{<:SegmentEvent{T}}},
+    ; atol::AbstractFloat=default_atol, rtol::AbstractFloat=default_rtol, fuzzy_rtol::AbstractFloat=1.0
+    ) where T
+    base_annotated_segments = event_loop!(subject; self_intersection=true, atol=atol, rtol=rtol)
+    for polygon in polygons
+        annotated_segments = event_loop!(polygon; self_intersection=true, atol=atol, rtol=rtol)
+        queue = SegmentEvent{T}[]
+        for ev in vcat(base_annotated_segments, annotated_segments)
+            add_annotated_segment!(queue, ev)
+        end
+        combined_annotated_segments = event_loop!(queue; self_intersection=false, atol=atol, rtol=rtol)
+        # for consistent reporting, swap annotations so that self annotations are always the primary
+        for ev in combined_annotated_segments
+            if !ev.primary
+                temp = ev.self_annotations
+                ev.self_annotations = ev.other_annotations
+                ev.other_annotations = temp
+            end
+        end
+        base_annotated_segments = apply_selection_criteria(combined_annotated_segments, selection_criteria)
+    end
+    empty_segments, region_segments = separate(is_empty_segment, base_annotated_segments)
+    regions = chain_segments(region_segments; atol=atol, check_closes=true, fuzzy_rtol=fuzzy_rtol)
     segment_chains = chain_segments(empty_segments; atol=atol, check_closes=false)
     # It is also possible to attach some segment_chains to regions.
     # This will give consistent results with the Weiler-Atherton implementation.
@@ -139,59 +156,6 @@ function martinez_rueda_algorithm(
     vcat(regions, segment_chains)
 end
 
-function insert_in_order!(vec::Vector{T}, data::T; lt=isless, rev=false) where T
-    idx = searchsortedfirst(vec, data; lt=lt, rev=rev)
-    insert!(vec, idx, data)
-end
-
-function pop_key!(vec::Vector{T}, key::T) where T
-    idx = findfirst(x->x===key, vec)
-    if isnothing(idx)
-        throw(KeyError(key))
-    end
-    popat!(vec, idx)
-end
-
-#############################################################
-##                  Initialise Events                      ##
-#############################################################
-
-function convert_to_event_queue(polygon::Polygon2D{T}; primary::Bool=true, atol::AbstractFloat=1e-6) where T
-    # The event list reads all segments from left to right, end->start, top to bottom
-    queue = SegmentEvent{T}[]
-    pt2 = polygon[end]
-    for i in eachindex(polygon)
-        pt1 = pt2
-        pt2 = polygon[i]
-        forward = _compare_points(pt1, pt2; atol=atol)
-        if forward == 0
-            continue # zero length segment
-        end
-        start = forward < 0 ? pt1 : pt2
-        end_ = forward < 0 ? pt2 : pt1
-        segment = (start, end_)
-        add_segment_event!(queue, segment, primary)
-    end
-    queue
-end
-
-function add_segment_event!(
-    queue::Vector{<:SegmentEvent},
-    segment::Segment2D,
-    primary::Bool,
-    shared_self_annotations::SegmentAnnotations=SegmentAnnotations(),
-    shared_other_annotations::SegmentAnnotations=SegmentAnnotations(),
-    )
-    forward = _compare_points(segment[1], segment[2])
-    @assert forward < 0 "invalid segment $(segment). Require start to be to the left or directly below the end."
-    start_event = SegmentEvent(segment, true, primary, shared_self_annotations, shared_other_annotations)
-    end_event = SegmentEvent(segment, false, primary, shared_self_annotations, shared_other_annotations)
-    start_event.other = end_event
-    end_event.other = start_event   
-    insert_in_order!(queue, start_event; lt=compare_events)
-    insert_in_order!(queue, end_event; lt=compare_events)
-end
-
 function add_annotated_segment!(queue::Vector{<:SegmentEvent}, ev::SegmentEvent)
     pt1 = ev.segment[1]
     pt2 = ev.segment[2]
@@ -205,62 +169,13 @@ function add_annotated_segment!(queue::Vector{<:SegmentEvent}, ev::SegmentEvent)
     add_segment_event!(queue, segment, ev.primary, ev.self_annotations, ev.other_annotations)
 end
 
-"""
-    _compare_points(pt1, pt2)
-Return:
-- -1 if pt1 is smaller
-- 0 if the same
-- 1 if pt2 is smaller
-"""
-function _compare_points(pt1::Point2D{T}, pt2::Point2D{T}; atol::AbstractFloat=1e-6) where T # pointsCompare
-    if abs(pt1[1] - pt2[1]) < atol # on a vertical line
-        if abs(pt1[2] - pt2[2]) < atol # same point
-            return 0
-        end
-        return pt1[2] < pt2[2] ? -1 : 1; # compare Y values
-    end
-    return pt1[1] < pt2[1] ? -1 : 1; # compare X values
-end
-
-"""
-    compare_events(event, here)
-
-Smaller events are to the left or bottom. Otherwise, end events come before the start.
-Returns true if smaller.
-"""
-function compare_events(event::SegmentEvent, here::SegmentEvent; atol::AbstractFloat=1e-6) # eventCompare
-    # Assumes events are left to right
-    comp = _compare_points(event.point, here.point)
-    if comp != 0
-        return comp < 0
-    end
-    # Selected points are the same -> events on top of each other.
-    comp = _compare_points(event.other_point, here.other_point)
-    if comp === 0
-        return false # equal segments
-    end
-    # Two events on top of each other.
-    if event.is_start != here.is_start
-        # favor the one that isn't the start
-        return event.is_start ? false : true
-    end
-    # share a common start point ⋅< or a common end point >⋅
-    # Manually calculate if the other point is above
-    if abs(here.segment[1][1] - here.segment[2][1]) < atol # vertical
-        # projecting the point won't work.
-        # instead, assume smaller segment leans towards the right
-        return event.other_point[1] > here.segment[1][1]
-    end
-    is_above_or_on(event.other_point, here.segment) ? false : true
-end
-
 #############################################################
 ##                    Event Loop                           ##
 #############################################################
 
 function event_loop!(
     queue::Vector{SegmentEvent{T}}
-    ; self_intersection::Bool, atol::AbstractFloat=1e-6, rtol::AbstractFloat=1e-4
+    ; self_intersection::Bool, atol::AbstractFloat=default_atol, rtol::AbstractFloat=default_atol
     ) where T # eventLoop
     annotated_segments = SegmentEvent{T}[]
     sweep_status = SegmentEvent{T}[] # current events in a vertical line, top to bottom.
@@ -270,7 +185,7 @@ function event_loop!(
         status_length = length(sweep_status)
         @debug("[event_loop!] ($(queue_length), $(status_length)): $(head)")
         if head.is_start # then check for intersections and add to sweep status
-            idx = find_transition(sweep_status, head; atol=atol, rtol=rtol)
+            idx = find_transition(sweep_status, head; atol=atol)
             above = idx == 1 ? nothing : sweep_status[idx - 1]
             below = (idx > length(sweep_status)) ? nothing : sweep_status[idx]
             @debug("[event_loop!] transition idx=$idx")
@@ -291,7 +206,7 @@ function event_loop!(
             end
             insert!(sweep_status, idx, head)
         else # event is ending, so remove it from the status
-            idx = find_transition(sweep_status, head.other; atol=atol, rtol=rtol)
+            idx = find_transition(sweep_status, head.other; atol=atol)
             if !(0 < idx <= length(sweep_status) && sweep_status[idx] === head.other)
                 @warn "$(head.other) was not in the expected location in the sweep status. " * 
                     "Falling back to linear search. This might result in incorrect annotations and hence open chains."
@@ -307,7 +222,7 @@ function event_loop!(
                     queue, sweep_status[idx - 1], sweep_status[idx + 1], self_intersection
                     ; atol=atol, rtol=rtol)
             end
-            push!(annotated_segments, copy_segment(head.other))
+            push!(annotated_segments, copy_segment(head.other, head.other.primary))
             popat!(sweep_status, idx)
         end
         popfirst!(queue)
@@ -315,53 +230,9 @@ function event_loop!(
     annotated_segments
 end
 
-function find_transition(
-    list::Vector{<:SegmentEvent}, event::SegmentEvent
-    ; atol::AbstractFloat=1e-6, rtol::AbstractFloat=1e-4
-    )
-    searchsortedfirst(list, event; lt=(x, y) -> is_above(x, y; atol=atol, rtol=rtol))
-end
-
-"""
-    is_above(event, other, [atol, rtol])
-
-
-!!!!! Critical function. May be source of errors that only emerge later.
-
-Return `true` if `event` is strictly above `other`.
-
-A segment is considered above another if:
-    1. It is to the left of the other segment.
-    2. And the start point of the other segment is orientated clockwise from it.
-Or symmetrically:
-    1. It is in line or to the right of the other segment.
-    2. And its start point is orientated counter-clockwise from the other segment.
-
-Assumes segments always go left to right.
-"""
-function is_above(
-    ev::SegmentEvent, other::SegmentEvent
-    ; atol::AbstractFloat=1e-6, rtol::AbstractFloat=1e-4
-    ) # statusCompare
-    seg1 = ev.segment
-    seg2 = other.segment
-    if (seg1[1][1] < seg2[1][1])
-        orient = get_orientation(seg1[1], seg1[2], seg2[1]; rtol=rtol, atol=atol)
-        if orient == COLINEAR
-            orient = get_orientation(seg1[1], seg1[2], seg2[2]; rtol=rtol, atol=atol)
-        end
-        return orient == CLOCKWISE
-    else
-        orient = get_orientation(seg2[1], seg2[2], seg1[1]; rtol=rtol, atol=atol)
-        if orient == COLINEAR
-            orient = get_orientation(seg2[1], seg2[2], seg1[2]; rtol=rtol, atol=atol)
-        end
-        return orient == COUNTER_CLOCKWISE
-    end
-end
-
 function check_and_divide_intersection!(
-    queue::Vector{<:SegmentEvent}, ev1::SegmentEvent, ev2::Nothing, self_intersection::Bool; atol=1e-6, rtol=1e-4
+    queue::Vector{<:SegmentEvent}, ev1::SegmentEvent, ev2::Nothing, self_intersection::Bool
+    ; atol::AbstractFloat=default_atol, rtol::AbstractFloat=default_rtol
     )
     queue
 end
@@ -371,13 +242,14 @@ function check_and_divide_intersection!(
     ev1::SegmentEvent,
     ev2::SegmentEvent,
     self_intersection::Bool
-    ; atol::AbstractFloat=1e-6, rtol::AbstractFloat=1e-4
+    ; atol::AbstractFloat=default_atol, rtol::AbstractFloat=default_rtol
     )
-    pt = intersect_geometry(ev1.segment, ev2.segment)
+    pt = intersect_geometry(ev1.segment, ev2.segment; atol=atol, rtol=rtol)
     if isnothing(pt)
-        # Lines need to be exactly on top of each other 
-        ori2_start = get_orientation(ev1.segment[1], ev1.segment[2], ev2.segment[1]; rtol=rtol)
-        ori2_end = get_orientation(ev1.segment[1], ev1.segment[2], ev2.segment[2]; rtol=rtol)
+        @debug("no intersection or parallel lines at $ev1 -- $ev2")
+        # Lines might be on top of each other 
+        ori2_start = get_orientation(ev1.segment[1], ev1.segment[2], ev2.segment[1]; atol=atol)
+        ori2_end = get_orientation(ev1.segment[1], ev1.segment[2], ev2.segment[2]; atol=atol)
         if (ori2_start == COLINEAR) && (ori2_end == COLINEAR)
             divide_coincident_intersection!(queue, ev1, ev2, self_intersection; atol=atol)
         end
@@ -396,7 +268,7 @@ function divide_intersection!(
     ev1::SegmentEvent,
     ev2::SegmentEvent,
     pt::Point2D
-    ; atol::AbstractFloat=1e-6
+    ; atol::AbstractFloat=default_atol
     ) # checkIntersection
     @debug("[divide_intersection!] $(ev1.segment) -- $(ev2.segment) at $(pt)")
     at_start1, at_end1, along1 = classify_intersection(ev1.segment, pt; atol=atol)
@@ -424,7 +296,7 @@ end
 
 function divide_coincident_intersection!(
     queue::Vector{<:SegmentEvent}, ev1::SegmentEvent, ev2::SegmentEvent, self_intersection::Bool
-    ; atol::AbstractFloat=1e-6
+    ; atol::AbstractFloat=default_atol
     )
     # This assumes:
     # - ev1 is on top of or to the right of ev2, because events are processed left to right.
@@ -490,7 +362,7 @@ Divide an event `ev` and `ev.other` in `queue` into 4:
 --x-->  to  --> x-->
 ```
 """
-function divide_event!(queue::Vector{<:SegmentEvent}, ev::SegmentEvent, pt::Point2D; atol::AbstractFloat=1e-6) # eventDivide
+function divide_event!(queue::Vector{<:SegmentEvent}, ev::SegmentEvent, pt::Point2D; atol::AbstractFloat=default_atol) # eventDivide
     # assumes pt lies on ev.segment
     new_segment = (pt, ev.segment[2])
     @debug("[divide_event!] new_segment=$(new_segment)")
@@ -511,7 +383,7 @@ Slides an end backwards.
     (start)---(end)
 ```
 """
-function update_end!(ev::SegmentEvent, end_point::Point2D; atol::AbstractFloat=1e-6)
+function update_end!(ev::SegmentEvent, end_point::Point2D; atol::AbstractFloat=default_atol)
     # Assumes ev is a start event.        
     @assert ev.is_start
     ev.segment = (ev.segment[1], end_point)
@@ -561,7 +433,7 @@ function calculate_self_annotations!(ev::SegmentEvent, below::Union{Nothing, Seg
     @debug("[calculate_self_annotations!] below: $(below)")
     toggle = isnothing(ev.self_annotations.fill_below) ? true : ev.self_annotations.fill_above != ev.self_annotations.fill_below
     if isnothing(below)
-        ev.self_annotations.fill_below = false # TODO primaryPolyInverted
+        ev.self_annotations.fill_below = false
     else
         @assert !isnothing(below.self_annotations.fill_above) "missing annotations below: $(below)" # preempt !nothing error
         ev.self_annotations.fill_below = below.self_annotations.fill_above # below should already be filled
@@ -579,8 +451,8 @@ function calculate_other_annotations!(ev::SegmentEvent, below::Nothing)
     @debug("[calculate_other_annotations!] ev=$ev")
     @debug("[calculate_other_annotations!] below=$below")
     if isnothing(ev.other_annotations.fill_above)
-        # if nothing is below ev, only inside if the other polygon is inverted
-        inside = false # TODO is_primary ? secondaryPolyInverted : primaryPolyInverted
+        # if nothing is below the event, it cannot be in the other polygon
+        inside = false
         ev.other_annotations.fill_above = inside
         ev.other_annotations.fill_below = inside
     end
@@ -672,12 +544,12 @@ function apply_selection_criteria(annotated_segments::Vector{<:SegmentEvent{T}},
                 (ev.other_annotations.fill_above ? 2 : 0) + 
                 (ev.other_annotations.fill_below ? 1 : 0)
         if criteria[index] != BLANK
-            new_segment = copy_segment(ev)
+            new_segment = copy_segment(ev, true)
             new_segment.self_annotations.fill_above = criteria[index] == ABOVE
             new_segment.self_annotations.fill_below = criteria[index] == BELOW
             new_segment.other_annotations.fill_above = nothing
             new_segment.other_annotations.fill_below = nothing
-            push!(result, copy_segment(new_segment))
+            push!(result, new_segment)
         end
     end
     result
@@ -691,20 +563,18 @@ is_empty_segment(ev::SegmentEvent) = (ev.self_annotations.fill_above == false) &
 struct SegmentChainCandidate{T}
     chain_idx::Int
     match_chain_start::Bool
-    match_segment_start::Bool
-    match_point::Point2D{T}
-    other_point::Point2D{T}
+    segment_event::SegmentEvent{T}
 end
 
 function chain_segments(
     segments::AbstractVector{SegmentEvent{T}}
-    ; atol::AbstractFloat=1e-6, check_closes::Bool=true
+    ; atol::AbstractFloat=default_atol, check_closes::Bool=true, fuzzy_rtol::AbstractFloat=1.0
     ) where T
     # Note: if any of the regions intersect at a vertex, than this is not guaranteed to give consistent results
     # They might be joined into one region or presented as separate regions.
     # This algorithm can fail if the polygon is improper (it has lines jutting out)
-    chains = Vector{Point2D{T}}[] # this is the same type as regions
-    regions = Vector{Point2D{T}}[]
+    chains = Vector{SegmentEvent{T}}[]
+    regions = Vector{SegmentEvent{T}}[]
     processed = Set{Segment2D{T}}()
     for event in segments
         if event.segment in processed
@@ -715,10 +585,13 @@ function chain_segments(
         @debug("[chain_segment]: event=$event")
         @debug("[chain_segment]: candidates=$candidates")
         for (chain_idx, chain) in enumerate(chains)
-            insert_matching_candidate!(candidates, chain, chain_idx, event.segment; atol=atol)
+            insert_matching_candidate!(candidates, chain, chain_idx, event; atol=atol)
         end
         if length(candidates) == 0 # start a new open chain
-            chain = [event.segment[1], event.segment[2]]
+            chain = [
+                SegmentEvent(event.segment, true, true, deepcopy(event.self_annotations)),
+                SegmentEvent(event.segment, false, true, deepcopy(event.self_annotations))
+            ]
             @debug("[chain_segment]: new chain")
             push!(chains, chain)
         elseif length(candidates) == 1 # check if it closes else append to chain
@@ -735,7 +608,6 @@ function chain_segments(
         elseif length(candidates) == 2 # join two chains together
             cand1 = candidates[1]
             cand2 = candidates[2]
-            @assert cand1.match_segment_start != cand2.match_segment_start "Same point of segment $(cand1.segment) linked to two open chains"
             chain1 = chains[cand1.chain_idx]
             chain2 = chains[cand2.chain_idx]
             append_candidate!(chain1, cand1)
@@ -751,7 +623,7 @@ function chain_segments(
     # - it is improper: the beginning and end is a segment(s) jutting out, so it can be closed with a segment in processing
     if check_closes
         if !isempty(chains)
-            fuzzy_close!(chains, regions; atol=atol)
+            fuzzy_close!(chains, regions; atol=atol, rtol=fuzzy_rtol)
         end
         @assert isempty(chains) "There are still open chains at the end of processing all segments."
         return regions
@@ -762,81 +634,111 @@ end
 
 function insert_matching_candidate!(
     candidates::Vector{<:SegmentChainCandidate},
-    chain::Vector{<:Point2D},
+    chain::Vector{<:SegmentEvent},
     chain_idx::Int,
-    segment::Segment2D
-    ; atol::AbstractFloat=1e-6
+    event::SegmentEvent
+    ; atol::AbstractFloat=default_atol
     )
+    segment = event.segment
     is_match = false
-    if is_same_point(chain[1], segment[1]; atol=atol)
+    if is_same_point(chain[1].point, segment[1]; atol=atol)
         is_match = true
         match_chain_start = true
         match_idx = 1
-        other_idx = 2
-    elseif is_same_point(chain[1], segment[2]; atol=atol)
+    elseif is_same_point(chain[1].point, segment[2]; atol=atol)
         is_match = true
         match_chain_start = true
         match_idx = 2
-        other_idx = 1
-    elseif is_same_point(chain[end], segment[1]; atol=atol)
+    elseif is_same_point(chain[end].point, segment[1]; atol=atol)
         is_match = true
         match_chain_start = false
         match_idx = 1
-        other_idx = 2
-    elseif is_same_point(chain[end], segment[2]; atol=atol)
+    elseif is_same_point(chain[end].point, segment[2]; atol=atol)
         is_match = true
         match_chain_start = false
         match_idx = 2
-        other_idx = 1
     end
     if is_match
+        is_start = match_idx == 1
         candidate = SegmentChainCandidate(
-            chain_idx, match_chain_start, match_idx == 1, segment[match_idx], segment[other_idx]
+            chain_idx,
+            match_chain_start,
+            # add segment for the other point
+            SegmentEvent(segment, !is_start, true, deepcopy(event.self_annotations))
         )
         push!(candidates, candidate)
     end
 end
 
-function append_candidate!(chain::Vector{<:Point2D}, candidate::SegmentChainCandidate)
+function append_candidate!(
+    chain::Vector{<:SegmentEvent},
+    candidate::SegmentChainCandidate
+    ; atol::AbstractFloat=default_atol
+    )
     if candidate.match_chain_start
-        insert!(chain, 1, candidate.other_point)
+        if length(chain) > 1 && 
+            get_orientation(
+                candidate.segment_event.point,
+                chain[1].point,
+                chain[2].point
+                ; atol=atol
+            ) == COLINEAR
+            popfirst!(chain)
+        end
+        insert!(chain, 1, candidate.segment_event)
     else
-        push!(chain, candidate.other_point)
+        if length(chain) > 1 && 
+            get_orientation(
+                chain[end-1].point,
+                chain[end].point,
+                candidate.segment_event.point
+                ; atol=atol
+            ) == COLINEAR
+            pop!(chain)
+        end
+        push!(chain, candidate.segment_event)
     end
 end
 
-function closes_chain(chain::Vector{<:Point2D}, candidate::SegmentChainCandidate; atol::AbstractFloat=1e-6)
+function closes_chain(chain::Vector{<:SegmentEvent}, candidate::SegmentChainCandidate; atol::AbstractFloat=default_atol)
+    candidate_point = candidate.segment_event.point
     if candidate.match_chain_start
-        return is_same_point(chain[end], candidate.other_point; atol=atol)
+        return is_same_point(chain[end].point, candidate_point; atol=atol)
     else
-        return is_same_point(chain[1], candidate.other_point; atol=atol)
+        return is_same_point(chain[1].point, candidate_point; atol=atol)
     end
 end
 
-function fuzzy_close!(chains::Vector{<:Vector{<:Point2D}}, regions::Vector{<:Vector{<:Point2D}}; atol)
+function fuzzy_close!(
+    chains::Vector{<:Vector{<:SegmentEvent}},
+    regions::Vector{<:Vector{<:SegmentEvent}}
+    ; atol::AbstractFloat, rtol::AbstractFloat=1.0
+    )
     for idx in reverse(eachindex(chains))
-        if is_fuzzy_closed(chains[idx], length(regions) + 1; atol=atol)
+        if is_fuzzy_closed(chains[idx], length(regions) + 1; atol=atol, rtol=rtol)
             push!(regions, popat!(chains, idx))
         end
     end
     regions
 end
 
-function is_fuzzy_closed(chain::Vector{<:Point2D}, idx::Int; atol::AbstractFloat, rtol::AbstractFloat=1.0)
-    if is_same_point(chain[1], chain[end]; atol=atol)
+function is_fuzzy_closed(chain::Vector{<:SegmentEvent}, idx::Int; atol::AbstractFloat, rtol::AbstractFloat=1.0)
+    if is_same_point(chain[1].point, chain[end].point; atol=atol)
         return true
     end
-    gap = norm(chain[1], chain[end])
-    gaps = norm.(chain[1:(end-1)], chain[2:end])
+    gap = norm(chain[1].point, chain[end].point)
+    path = map(event -> event.point, chain)
+    gaps = norm.(path[1:(end-1)], path[2:end])
     mean_gap = sum(gaps) / length(gaps)
     if gap / mean_gap <= rtol
-        @warn("Region $idx was not closed, but it has a relatively small gap and will be considered closed.")
+        @warn("Region $idx was not closed, but it has a relatively small gap and will be considered closed.
+        |gap| / |mean_segment| = $gap / $mean_gap < $rtol.")
         return true
     end
     false
 end
 
-function join_chains!(chain1::Vector{<:Point2D}, chain2::Vector{<:Point2D}, match_chain1_start, match_chain2_start)
+function join_chains!(chain1::Vector{<:SegmentEvent}, chain2::Vector{<:SegmentEvent}, match_chain1_start, match_chain2_start)
     # Note: with clever use of reverse! can change this to always modify chain1 in place for the same output
     if match_chain1_start && match_chain2_start
         # <--- --->
@@ -851,4 +753,97 @@ function join_chains!(chain1::Vector{<:Point2D}, chain2::Vector{<:Point2D}, matc
         # ----> <-----
         return push!(chain1, reverse!(chain2)...) 
     end
-end
\ No newline at end of file
+end
+
+#############################################################
+##                  Polygons and Holes                     ##
+#############################################################
+
+function convert_segments_to_polygons(
+    regions::Vector{<:Vector{<:SegmentEvent}}
+    ; atol::AbstractFloat=default_rtol
+    )
+    candidates, exteriors  = separate(p -> is_hole(p; atol=atol), regions)
+    polygons = map(segments -> Polygon(map(event -> event.point, segments)), exteriors)
+    holes = map(segments -> map(event -> event.point, segments), candidates)
+    parents = match_holes_polygons(polygons, holes; atol=atol)
+    for (idx, hole) in zip(parents, holes)
+        if idx == 0
+            compact_vec = "[$(hole[1])...$(hole[end])]"
+            @debug "Hole $(compact_vec) has no parent. Casting to Polygon."
+            push!(polygons, Polygon(hole))
+        else
+            push!(polygons[idx].holes, hole)
+        end
+    end
+    polygons
+end
+
+"""
+    is_hole(polygon::Vector{<:SegmentEvent}; atol=default_atol)
+
+A necessary and sufficient condition for a polygon to be classified as a hole is that
+at its lowest point it must be filled below and not above.
+
+The `self_annotations` must therefore not be `nothing`.
+
+Note: if the direction of the polygon was known (clockwise/counter-clockwise) then any point could be used.
+"""
+function is_hole(polygon::Vector{<:SegmentEvent}; atol::AbstractFloat=default_atol)
+    # instead of sorting the whole vector, get the lowest segments first
+    y = minimum(event -> event.point[2], polygon)
+    lowest_segments = filter(event -> event.point[2] == y || event.other_point[2] == y, polygon)
+    # sort and check annotations
+    sort!(lowest_segments, lt=(x, y) -> is_above(x, y; atol=atol))
+    annotations = lowest_segments[end].self_annotations
+    annotations.fill_below && !annotations.fill_above
+end
+
+"""
+    match_holes_polygons(polygons::Vector{<:Polygon}, holes::Vector{<:Tuple})
+
+An algorithm for matching holes to polygons.
+Returns the index of each parent for each hole.
+
+For every hole, match to a polygon that contains the hole.
+If there are multiple polygons possible, the polygon with the least area is chosen.
+If no polygons are found, return the hole as a `Polygon`.
+
+In the best case there is one polygon or one hole. Then this runs in `O(1)` time.
+In the worst case, none of the holes match to a polygon.
+Then this runs in `O(phn)` time where `p` is the number polygons,
+`h` is the number of holes and `n` is the average number of vertices defining each polygon.
+"""
+function match_holes_polygons(
+    polygons::Vector{<:Polygon},
+    holes::Vector{<:Path2D}
+    ; atol::AbstractFloat=default_atol
+    )
+    if length(polygons) == 1
+        return fill(1, length(holes))
+    elseif isempty(holes)
+        return Int[]
+    end
+    areas = map(area_polygon, polygons)
+    # sort by ascending areas. Therefore smallest parent is matched first.
+    # Trade off is the hole may be tried in many smaller polygons first.
+    idxs = sortperm(areas)
+    parents = zeros(Int, length(holes))
+    for (idx_h, candidate) in enumerate(holes)
+        found = false
+        for (idx_p, parent) in zip(idxs, polygons[idxs])
+            # Assume that no segments intersect after the Martínez-Rueda algorithm.
+            # Then only need to check a point not on the exterior.
+            j = 1
+            while (j < length(candidate)) && on_border(parent.exterior, candidate[j])
+                j += 1
+            end
+            found = contains(parent.exterior, candidate[j]; atol=atol, on_border_is_inside=false)
+            if found
+                parents[idx_h] = idx_p
+                break
+            end
+        end
+    end
+    parents
+end
