@@ -15,9 +15,9 @@ using PolygonAlgorithms: PointSearchAlg, ChasingEdgesAlg, WeilerAthertonAlg, Mar
     poly2 = [
         (1.0, 1.0), (1.0, 2.0), (2.0, 2.0), (2.0, 1.0)
     ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(poly2)
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(poly2)
 end
 
@@ -25,7 +25,7 @@ end
     poly1 = [
         (0.0, 0.0), (0.0, 2.0), (2.0, 2.0), (2.0, 0.0)
     ]
-    points = intersect_convex(poly1, poly1, alg)
+    points = intersect_convex(alg, poly1, poly1)
     @test PointSet(points) == PointSet(poly1)
 
     # overlap
@@ -33,13 +33,13 @@ end
     expected = [
         (1.0, 1.0), (1.0, 2.0), (2.0, 2.0), (2.0, 1.0), 
     ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
 
     # no intersection
     poly2 = translate(poly1, (4.0, 3.0))
     expected = []
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test isempty(points)
 end
 
@@ -52,9 +52,9 @@ end
     expected = [
         (2.0, 1.0), (2.0, 2.0)
     ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 
     poly2 = [
@@ -63,9 +63,9 @@ end
     expected = [
         (2.0, 0.0), (2.0, 2.0)
     ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 
     # edge + vertex overlap
@@ -73,9 +73,9 @@ end
     expected = [
         (1.0, 2.0), (2.0, 2.0), (2.0, 0.0), (1.0, 0.0)
         ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 end
 
@@ -87,10 +87,10 @@ end
     poly2 = translate(poly1, (2.0, 2.0))
     expected = (typeof(alg) == PolygonAlgorithms.MartinezRuedaAlg) ?
         Tuple{Float64, Float64}[] : [(2.0, 2.0)]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
 
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 end
 
@@ -107,10 +107,10 @@ end
         (2.0, 3.0),
         (4.222222, 1.296296),
     ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
 
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 end
 
@@ -134,11 +134,11 @@ end
         (0.171429, 0.528571),
         (0.7, 0.4),
     ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     answer = Set([round.(p, digits=6) for p in points])
     @test issetequal(answer, expected)
 
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     answer = Set([round.(p, digits=6) for p in points])
     @test issetequal(answer, expected)
 end
@@ -156,10 +156,10 @@ end
         (2.0, 3.0),
         (4.545455, 0.757576),
     ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
 
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 end
 
@@ -174,25 +174,25 @@ end
     # single point
     expected = (typeof(alg) == PolygonAlgorithms.MartinezRuedaAlg) ?
         Tuple{Float64, Float64}[] : [(0.5, 1.0)]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 
     # 2 points inside
     poly1_ = translate(poly1, ((0.0), (1.5)))
     expected = poly1_
-    points = intersect_convex(poly1_, poly2, alg)
+    points = intersect_convex(alg, poly1_, poly2)
     @test PointSet(points) == PointSet(expected)
-    points = intersect_convex(poly2, poly1_, alg)
+    points = intersect_convex(alg, poly2, poly1_)
     @test PointSet(points) == PointSet(expected)
 
     # 3 points inside
     poly1_ = translate(poly1, ((0.0), (2.0)))
     expected = poly1_
-    points = intersect_convex(poly1_, poly2, alg)
+    points = intersect_convex(alg, poly1_, poly2)
     @test PointSet(points) == PointSet(expected)
-    points = intersect_convex(poly2, poly1_, alg)
+    points = intersect_convex(alg, poly2, poly1_)
     @test PointSet(points) == PointSet(expected)
 end
 
@@ -205,9 +205,9 @@ end
     ]
 
     expected = [(4.0, 2.0), (3.0, 4.0), (4.666666666666667, 4.0)]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 end
 
@@ -219,22 +219,23 @@ end
         (5.0, 1.0), (1.0, 4.0), (9.0, 4.0)
     ]
     expected = [(1.0, 4.0), (9.0, 4.0)]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 
-    # shared point
+    # extra shared point on line
     poly1 = [
         (5.0, 8.0), (9.0, 4.0), (5.0, 4.0), (1.0, 4.0),
     ]
     poly2 = [
         (5.0, 1.0), (1.0, 4.0), (5.0, 4.0), (9.0, 4.0)
     ]
-    expected = [(1.0, 4.0), (5.0, 4.0), (9.0, 4.0), (5.0, 4.0),]
-    points = intersect_convex(poly1, poly2, alg)
+    expected = (typeof(alg) == PolygonAlgorithms.MartinezRuedaAlg) ?
+        [(1.0, 4.0), (9.0, 4.0)] : [(1.0, 4.0), (5.0, 4.0), (9.0, 4.0), (5.0, 4.0),]
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 end
 
@@ -248,10 +249,10 @@ end
     expected = [
         (0.0, 1.5), (0.0, 0.5), (1.0, 1.5), (1.0, 0.5)
     ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
 
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 end
 
@@ -270,10 +271,10 @@ end
         (-1/3, h/3), (+1/3, h/3), 
         (-1/3, -h/3), (+1/3, -h/3), 
     ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     @test PointSet(points) == PointSet(expected)
 
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     @test PointSet(points) == PointSet(expected)
 end
 
@@ -309,11 +310,11 @@ end
         (6.0, 7.0),
         (1.0, 4.75),
     ]
-    points = intersect_convex(poly1, poly2, alg)
+    points = intersect_convex(alg, poly1, poly2)
     answer = Set([round.(p, digits=6) for p in points])
     @test PointSet(points) == PointSet(expected)
 
-    points = intersect_convex(poly2, poly1, alg)
+    points = intersect_convex(alg, poly2, poly1)
     answer = Set([round.(p, digits=6) for p in points])
     @test PointSet(points) == PointSet(expected)
 end
