@@ -1,6 +1,6 @@
 using Test
 using PolygonAlgorithms
-using PolygonAlgorithms: translate, rotate, PointSet
+using PolygonAlgorithms: translate, rotate, Point2D, PointSet
 
 function are_regions_equal(r1::Vector{<:Vector{<:Point2D}}, r2::Vector{<:Vector{<:Point2D}})
     if length(r1) != length(r2)
@@ -12,14 +12,23 @@ function are_regions_equal(r1::Vector{<:Vector{<:Point2D}}, r2::Vector{<:Vector{
 end
 
 @testset verbose = true "PolygonAlgorithms" begin
-    include("linked_list.jl")
-    include("point_set.jl")
+    # data structures
+    include("data_structures/linked_list.jl")
+    include("data_structures/point_set.jl")
+    # segments
+    include("intersect.jl")
+    include("line_sweep.jl")
+    # polygon 
+    include("convex_hull.jl")
     include("area-centroid.jl")
     include("bounds.jl")
-    include("intersect.jl")
     include("point_in_polygon.jl")
-    include("martinez_rueda.jl")
+    include("data_structures/polygon.jl")
+    # boolean
+    include("boolean/martinez_rueda.jl")
     include("intersect_convex.jl")
     include("intersect_concave.jl")
-    include("convex_hull.jl")
+    include("polygon_boolean.jl")
+    include("polygon_boolean_holes.jl")
+    include("polygon_boolean_multi.jl")
 end
\ No newline at end of file
